%% generate isoform structure information from the processed bed file
clc;clear;close all;
addpath(strcat(pwd,'/codes'));
addpath(strcat(pwd,'/codes/strings'));
addpath(strcat(pwd,'/annotation_info'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% load .bed file
filename_bed = 'hg19.gene.post.bed';
[chromosome_all,start_pos_all,end_pos_all,gene_name_all,score_all,strand_all,thickStart_all,...
    thickEnd_all,itemRgb_all,blockCount_all,blockSizes_all,blockStarts_all]...
    =textread(filename_bed,'%s%d%d%s%s%s%d%d%s%d%s%s');
%% group genes according to the number of isoforms
A = tabulate(gene_name_all);
Pool_K = unique(cell2mat(A(:,2)));
Gene_ID = cell(length(Pool_K),1);
INDEX_isof_hg19 = cell(length(Pool_K),1);
for i = 1:length(Pool_K)
    Gene_ID{i} = A(cell2mat(A(:,2))==Pool_K(i),1);
    INDEX_isof_hg19{i} = [];
    for j = 1:length(Gene_ID{i})
        INDEX_isof_hg19{i} = [INDEX_isof_hg19{i};find(strcmp(gene_name_all,Gene_ID{i}{j}))];
    end
end

%% summarize isoform structure for each group of genes
geneName_all = cell(length(Pool_K),1);
isofName_all = cell(length(Pool_K),1);
isofStruct_all = cell(length(Pool_K),1);
for num_isoform = 1:length(Pool_K)
    fprintf('Processing genes with %d isoforms...\n',num_isoform);
    K = Pool_K(num_isoform);
    idx_iso = INDEX_isof_hg19{num_isoform};
    chromosome= chromosome_all(idx_iso);
    start_pos = start_pos_all(idx_iso);
    end_pos = end_pos_all(idx_iso);
    gene_name = gene_name_all(idx_iso);
    score = score_all(idx_iso);
    strand = strand_all(idx_iso);
    thickStart = thickStart_all(idx_iso);
    thickEnd = thickEnd_all(idx_iso);
    itemRgb = itemRgb_all(idx_iso);
    blockCount = blockCount_all(idx_iso);
    blockSizes = blockSizes_all(idx_iso);
    blockStarts = blockStarts_all(idx_iso);
    
    if K==1 %% genes with single isoform
        numGene=size(gene_name,1);
        exonLen=cell(numGene,1);
        geneLen=zeros(numGene,1);
        for i=1:numGene
            tmpLen=strsplit(blockSizes{i},',');
            for j=1:(length(tmpLen)-1)
                exonLen{i}(j,1)=str2num(tmpLen{j});
            end
            geneLen(i)=sum(exonLen{i});
        end
        geneName_all{num_isoform} = gene_name;
        isofName_all{num_isoform} = gene_name;
        isofStruct_all{num_isoform} = exonLen;
    else
        if sum(blockCount==0)>0
            idx_nan = (ceil(find(blockCount==0)/K)*K-(K-1)):ceil(find(blockCount==0)/K)*K;
            chromosome(idx_nan)= [];
            start_pos(idx_nan)= [];
            end_pos(idx_nan)= [];
            gene_name{num_isoform}(idx_nan)= [];
            score(idx_nan)= [];
            strand(idx_nan)= [];
            thickStart(idx_nan)= [];
            thickEnd(idx_nan)= [];
            itemRgb(idx_nan)= [];
            blockCount{num_isoform}(idx_nan)= [];
            blockSizes{num_isoform}(idx_nan)= [];
            blockStarts{num_isoform}(idx_nan)= [];
        end
        
        numIsof=size(gene_name,1);
        numGene=size(gene_name,1)/K;
        exonLen=cell(numIsof,1);
        exonStartRel=cell(numIsof,1);
        exonStartAbs=cell(numIsof,1);
        isofStruct_all{num_isoform}=cell(numGene,1);
        geneName_all{num_isoform}=gene_name(1:K:end);
        isofName_all{num_isoform}=cell(numGene,1);
        
        for i=1:numGene
            for j=1:K
                if j==1
                    isofName_all{num_isoform}{i}{j,1}=strcat(geneName_all{num_isoform}{i});
                else
                    isofName_all{num_isoform}{i}{j,1}=strcat(geneName_all{num_isoform}{i},'_dup',num2str(j-1));
                end
            end
        end
        
        for i=1:numIsof
            tmpLen=strsplit(blockSizes{i},',');
            tmpStart=strsplit(blockStarts{i},',');
            for j=1:(length(tmpLen)-1)
                exonLen{i}(j,1)=str2num(tmpLen{j});
                exonStartRel{i}(j,1)=str2num(tmpStart{j});
            end
            exonStartAbs{i}=exonStartRel{i}+start_pos(i);
        end
        for i=1:numGene
            unionExonStartAbs=[];
            for k=1:K
                unionExonStartAbs=union(exonStartAbs{K*(i-1)+k},unionExonStartAbs);
            end
            isofStruct_all{num_isoform}{i}=zeros(length(unionExonStartAbs),K);
            for k=1:K
                [common ia ib]=intersect(unionExonStartAbs,exonStartAbs{K*(i-1)+k});
                isofStruct_all{num_isoform}{i}(ia,k)=exonLen{K*(i-1)+k};
            end
            
        end
    end
end
    
save(strcat(pwd,'/annotation_info/Isof_structure_hg19.mat'),'geneName_all','isofName_all','isofStruct_all','INDEX_isof_hg19','Pool_K')
