function [beta_e1, d_e] = func_BayesIso(Y0, X0, I,G,K,J,J1,J2,Result_dir)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Poisson-Lognormal distribution estimation with random beta %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Parameter estimation...\n')
nsamples=1000;
thin=2;

%% initilization
Xm=cell2mat(X0);
s_Um = ones(size(Xm));
s_Um(Xm==0)=0;
s_U=mat2cell(s_Um,I,K);
s_betam=10*ones(K,J*G);
s_beta=mat2cell(s_betam,K, ones(G,1)*J);
s_beta=s_beta';
s_tau=1;

samples_beta=zeros(size(s_betam,1),size(s_betam,2),nsamples);
accp_matrix_beta=zeros(size(s_betam,1),size(s_betam,2),nsamples);
samples_U=zeros(size(s_Um,1),size(s_Um,2),nsamples);
accp_matrix_U=zeros(size(s_Um,1),size(s_Um,2),nsamples);
samples_tau=zeros(1,nsamples);

%%hyper-parameter
a=0.9;b=0.5;

for n=1:nsamples*thin
    if (n/nsamples)==floor(n/nsamples)
        fprintf('Estimating within-in sample overdispersion iteration %d\n',n);
    end
    %% 1.new sample for beta
    [sample_beta sample_accp_beta]=cellfun(@MS_beta, Y0, X0, s_U, s_beta,'UniformOutput',false);
    s_beta=sample_beta;
    
    %% 2. new sample for U
    sample_tau=ones(G,1)*s_tau;
    sample_tau=mat2cell(sample_tau,ones(G,1),1);
    [s_U sample_accp_U]=cellfun(@MS_U, Y0,X0,s_U,s_beta,sample_tau,'UniformOutput',false);
    s_Um=cell2mat(s_U);
    
    %% 3. new sample for tau
    s_tau=gamrnd(a+length(find(Xm>0))/2,1/(b+(sum(s_Um(Xm>0).^2)/2)));
    
    %% record samples
    if (n/thin)==floor(n/thin)
        samples_beta(:,:,floor(n/thin))=cell2mat(s_beta');
        accp_matrix_beta(:,:,floor(n/thin))=cell2mat(sample_accp_beta');
        samples_U(:,:,floor(n/thin))=cell2mat(s_U);
        accp_matrix_U(:,:,floor(n/thin))=cell2mat(sample_accp_U);
        samples_tau(:,floor(n/thin))=s_tau;
    end
end

U_e=mean(samples_U(:,:,floor(nsamples/2):end),3);
beta_e_PL=mean(samples_beta(:,:,floor(nsamples/2):end),3);
tau_e=mean(samples_tau(floor(nsamples/2):end));
beta_e_V=reshape(beta_e_PL,size(beta_e_PL,1)*size(beta_e_PL,2),1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% Estimation of parameters in Gamma-Gamma model with U_e
nsamples=500;
thin=5;
ite=nsamples*thin;
adaptive_nsamples=100;
adaptive_stage=5;
samples_beta=zeros(K,G*J,nsamples);
samples_beta_adapt=zeros(K,G*J,adaptive_nsamples*adaptive_stage);
accp_matrix_beta=zeros(K,G*J,nsamples);
accp_matrix_beta_adapt=zeros(K,G*J,adaptive_nsamples*adaptive_stage);
samples_lambda=zeros(K*G,2,nsamples);
samples_lambda_adapt=zeros(K*G,2,adaptive_nsamples*adaptive_stage);
samples_v=zeros(1,nsamples);
samples_v_adapt=zeros(1,adaptive_nsamples*adaptive_stage);
samples_d=zeros(G*K,nsamples);
samples_d_adapt=zeros(G*K,adaptive_nsamples*adaptive_stage);
samples_alpha0=zeros(1,nsamples);
samples_alpha0_adapt=zeros(1,adaptive_nsamples*adaptive_stage);
accp_matrix_alpha0=zeros(1,nsamples);
accp_matrix_alpha0_adapt=zeros(1,adaptive_nsamples*adaptive_stage);
samples_alpha=zeros(1,nsamples);
samples_alpha_adapt=zeros(1,adaptive_nsamples*adaptive_stage);
accp_matrix_alpha=zeros(1,nsamples);
accp_matrix_alpha_adapt=zeros(1,adaptive_nsamples*adaptive_stage);

% %%%%%%%%%%%%%%%%%%%
% %%% MS sampling %%%
% %%%%%%%%%%%%%%%%%%%
opt_accp_beta=0.4;
opt_accp_alpha0=0.4;
opt_accp_alpha=0.4;
prop_std_beta=0.5*ones(K,G*J);
prop_std_beta=mat2cell(prop_std_beta,K,ones(G,1)*J);
prop_std_alpha0=1;
prop_std_alpha=0.8;

%% initilization
s_beta=10*ones(K,G*J);
s_lambda=ones(K*G,2);
s_alpha=1;
s_alpha0=10;
s_v=10;
s_d=ones(G*K,1);
U_e=mat2cell(U_e,I,K);
%% hyper parameters
a=0.9;b=0.5;%% hyperparameters for v
for s=1:adaptive_stage
    fprintf('adaptive stage %d\n',s);
    for n=1:adaptive_nsamples
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %% 1. sample beta
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        sample_beta=mat2cell(s_beta,K,ones(G,1)*J);
        sample_beta=sample_beta';
        [sample_beta sample_accp_beta]=cellfun(@MS_beta_GG, Y0, X0, U_e, sample_beta,prop_std_beta','UniformOutput',false);
        s_beta=cell2mat(sample_beta');
        samples_beta_adapt(:,:,n+adaptive_nsamples*(s-1))=s_beta;
        accp_matrix_beta_adapt(:,:,n+adaptive_nsamples*(s-1))=cell2mat(sample_accp_beta');
        
        %%%%%%%%%%%%%%%%%%%%%
        %% 2. sample lambda
        %%%%%%%%%%%%%%%%%%%%%
        tmp_beta=mat2cell(s_beta,K,J*ones(G,1));
        tmp_beta=tmp_beta';
        tmp_beta=cell2mat(tmp_beta);
        %% non-differential genes
        id=find(s_d==0);
        a1=repmat(J*s_alpha+s_alpha0,length(id),1);
        b1=sum(tmp_beta(id,:),2)+s_v;
        s_lambda(id,:)=repmat(gamrnd(a1,1./b1),1,2);
        
        %% differential genes, group 1
        id=find(s_d==1);
        a1_1=repmat(J1*s_alpha+s_alpha0,length(id),1);
        b1_1=sum(tmp_beta(id,1:J1),2)+s_v;
        s_lambda(id,1)=gamrnd(a1_1,1./b1_1);
        %% differential genes, group 2
        a1_2=repmat(J2*s_alpha+s_alpha0,length(id),1);
        b1_2=sum(tmp_beta(id,J1+1:end),2)+s_v;
        s_lambda(id,2)=gamrnd(a1_2,1./b1_2);
        
        samples_lambda_adapt(:,:,n+adaptive_nsamples*(s-1))=s_lambda;
        
        %%%%%%%%%%%%
        %% 3. sample v
        %%%%%%%%%%%%
        [G1,s_lambda1,s_beta1,J0]=vectorize_params(s_d,s_lambda,tmp_beta,J1,J2);
        a1=G1*s_alpha0+a;
        b1=sum(s_lambda1)+b;
        s_v=gamrnd(a1,1/b1);
        samples_v_adapt(1,n+adaptive_nsamples*(s-1))=s_v;
        
        %%%%%%%%%%%%%%%%
        %% 4. sample d
        %%%%%%%%%%%%%%%%
        log_rho_func=inline('alpha0*log(v)+alogam(m*alpha+alpha0)+alogam(n*alpha+alpha0)-alogam((m+n)*alpha+alpha0)-alogam(alpha0)+((m+n)*alpha+alpha0)*log(v+yim+yin)-(m*alpha+alpha0)*log(v+yim)-(n*alpha+alpha0)*log(v+yin)','alpha0','alpha','m','n','v','yim','yin');
        
        log_rho=log_rho_func(s_alpha0,s_alpha,J1,J2,s_v,sum(tmp_beta(:,1:J1),2),sum(tmp_beta(:,J1+1:end),2));
        p1=ones(G*K,1)-1./(exp(log_rho)+1);
        tmp=rand(G*K,1);
        s_d=zeros(G*K,1);
        s_d(tmp<p1)=1;
        samples_d_adapt(:,n+adaptive_nsamples*(s-1))=s_d;
        
        %%%%%%%%%%%%%%%%%%
        %% 4. sample alpha0
        %%%%%%%%%%%%%%%%%%
        logfunc_alpha0=inline('(G*alpha0*log(v)+(alpha0-1)*sum(log(lambda))-G*log(gamma(alpha0)))','alpha0','v','lambda','G');
        logpdf_alpha0=@(x) logfunc_alpha0(x,s_v,s_lambda1,G1);
        
        %% using random walk proposal function
        proprnd_alpha0=@(x) abs(x+prop_std_alpha0*randn(1));
        logproppdf_alpha0=@(x,y) log(normpdf(x,y,prop_std_alpha0)+normpdf(x,-y,prop_std_alpha0));
        [s_alpha0 accp_alpha0]=mhsample(s_alpha0,1,'logpdf',logpdf_alpha0,'proprnd',proprnd_alpha0,'logproppdf',logproppdf_alpha0);
        
        samples_alpha0_adapt(1,n+adaptive_nsamples*(s-1))=s_alpha0;
        accp_matrix_alpha0_adapt(1,n+adaptive_nsamples*(s-1))=accp_alpha0;
        
        %%%%%%%%%%%%%%%%%%
        %% 5. sample alpha
        %%%%%%%%%%%%%%%%%%
        logfunc_alpha=inline('alpha*J*(log(lambda))+(alpha-1)*sum(sum(log(beta)))-sum(J)*alogam(alpha)','alpha','J','G','beta','lambda');
        logpdf_alpha=@(x) logfunc_alpha(x,J0',G1,tmp_beta,s_lambda1);
        
        %% using random walk proposal function
        proprnd_alpha=@(x) abs(x+prop_std_alpha*randn(1));
        logproppdf_alpha=@(x,y) log(normpdf(x,y,prop_std_alpha)+normpdf(x,-y,prop_std_alpha));
        [s_alpha accp_alpha]=mhsample(s_alpha,1,'logpdf',logpdf_alpha,'proprnd',proprnd_alpha,'logproppdf',logproppdf_alpha);
        
        samples_alpha_adapt(1,n+adaptive_nsamples*(s-1))=s_alpha;
        accp_matrix_alpha_adapt(1,n+adaptive_nsamples*(s-1))=accp_alpha;
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% tune the proposal scale for beta
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    current_accp_beta_adapt=mean(accp_matrix_beta_adapt(:,:,adaptive_nsamples*(s-1)+1:adaptive_nsamples*s),3);
    current_accp_beta_adapt(current_accp_beta_adapt==0)=1/adaptive_nsamples;
    current_accp_beta_adapt(current_accp_beta_adapt==1)=1-1/adaptive_nsamples;
    prop_std_beta=cell2mat(prop_std_beta);
    prop_std_beta=prop_std_beta*norminv(opt_accp_beta/2)./norminv(current_accp_beta_adapt/2);
    prop_std_beta=mat2cell(prop_std_beta,K,ones(G,1)*J);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% tune the proposal scale for alpha0
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    current_accp_alpha0_adapt=mean(accp_matrix_alpha0_adapt(1,adaptive_nsamples*(s-1)+1:adaptive_nsamples*s));
    current_accp_alpha0_adapt(current_accp_alpha0_adapt==0)=1/adaptive_nsamples;
    current_accp_alpha0_adapt(current_accp_alpha0_adapt==1)=1-1/adaptive_nsamples;
    prop_std_alpha0=prop_std_alpha0*norminv(opt_accp_alpha0/2)./norminv(current_accp_alpha0_adapt/2);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% tune the proposal scale for alpha
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    current_accp_alpha_adapt=mean(accp_matrix_alpha_adapt(1,adaptive_nsamples*(s-1)+1:adaptive_nsamples*s));
    current_accp_alpha_adapt(current_accp_alpha_adapt==0)=1/adaptive_nsamples;
    current_accp_alpha_adapt(current_accp_alpha_adapt==1)=1-1/adaptive_nsamples;
    prop_std_alpha=prop_std_alpha*norminv(opt_accp_alpha/2)./norminv(current_accp_alpha_adapt/2);
    
end
%%%%%%%%%%%%%%%%%%%%%%%%
%% stable sampling stage
%%%%%%%%%%%%%%%%%%%%%%%%
for n=1:ite
    if (n/nsamples)==floor(n/nsamples)
        fprintf('sampling stage: iteration # %d\n',n);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% sampling beta
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    sample_beta=mat2cell(s_beta,K,ones(G,1)*J);
    sample_beta=sample_beta';
    [sample_beta sample_accp_beta]=cellfun(@MS_beta_GG, Y0, X0, U_e, sample_beta,prop_std_beta','UniformOutput',false);
    s_beta=cell2mat(sample_beta');
    
    %%%%%%%%%%%%%%%%%%%%%
    %% 2. sample lambda
    %%%%%%%%%%%%%%%%%%%%%
    tmp_beta=mat2cell(s_beta,K,J*ones(G,1));
    tmp_beta=tmp_beta';
    tmp_beta=cell2mat(tmp_beta);
    %% non-differential genes
    id=find(s_d==0);
    a1=repmat(J*s_alpha+s_alpha0,length(id),1);
    b1=sum(tmp_beta(id,:),2)+s_v;
    s_lambda(id,:)=repmat(gamrnd(a1,1./b1),1,2);
    
    %% differential genes, group 1
    id=find(s_d==1);
    a1_1=repmat(J1*s_alpha+s_alpha0,length(id),1);
    b1_1=sum(tmp_beta(id,1:J1),2)+s_v;
    s_lambda(id,1)=gamrnd(a1_1,1./b1_1);
    %% differential genes, group 2
    a1_2=repmat(J2*s_alpha+s_alpha0,length(id),1);
    b1_2=sum(tmp_beta(id,J1+1:end),2)+s_v;
    s_lambda(id,2)=gamrnd(a1_2,1./b1_2);
    
    %%%%%%%%%%%%
    %% 3. sample v
    %%%%%%%%%%%%
    [G1,s_lambda1,s_beta1,J0]=vectorize_params(s_d,s_lambda,tmp_beta,J1,J2);
    a1=G1*s_alpha0+a;
    b1=sum(s_lambda1)+b;
    s_v=gamrnd(a1,1/b1);
    
    %%%%%%%%%%%%%%%%
    %% 4. sample d
    %%%%%%%%%%%%%%%%
    log_rho_func=inline('alpha0*log(v)+alogam(m*alpha+alpha0)+alogam(n*alpha+alpha0)-alogam((m+n)*alpha+alpha0)-alogam(alpha0)+((m+n)*alpha+alpha0)*log(v+yim+yin)-(m*alpha+alpha0)*log(v+yim)-(n*alpha+alpha0)*log(v+yin)','alpha0','alpha','m','n','v','yim','yin');
    
    log_rho=log_rho_func(s_alpha0,s_alpha,J1,J2,s_v,sum(tmp_beta(:,1:J1),2),sum(tmp_beta(:,J1+1:end),2));
    p1=ones(G*K,1)-1./(exp(log_rho)+1);
    tmp=rand(G*K,1);
    s_d=zeros(G*K,1);
    s_d(tmp<p1)=1;
    
    %%%%%%%%%%%%%%%%%%
    %% 4. sample alpha0
    %%%%%%%%%%%%%%%%%%
    logfunc_alpha0=inline('(G*alpha0*log(v)+(alpha0-1)*sum(log(lambda))-G*log(gamma(alpha0)))','alpha0','v','lambda','G');
    logpdf_alpha0=@(x) logfunc_alpha0(x,s_v,s_lambda1,G1);
    
    %% using random walk proposal function
    proprnd_alpha0=@(x) abs(x+prop_std_alpha0*randn(1));
    logproppdf_alpha0=@(x,y) log(normpdf(x,y,prop_std_alpha0)+normpdf(x,-y,prop_std_alpha0));
    [s_alpha0 accp_alpha0]=mhsample(s_alpha0,1,'logpdf',logpdf_alpha0,'proprnd',proprnd_alpha0,'logproppdf',logproppdf_alpha0);
    
    %%%%%%%%%%%%%%%%%%
    %% 5. sample alpha
    %%%%%%%%%%%%%%%%%%
    logfunc_alpha=inline('alpha*J*(log(lambda))+(alpha-1)*sum(sum(log(beta)))-sum(J)*alogam(alpha)','alpha','J','G','beta','lambda');
    logpdf_alpha=@(x) logfunc_alpha(x,J0',G1,tmp_beta,s_lambda1);
    
    %% using random walk proposal function
    proprnd_alpha=@(x) abs(x+prop_std_alpha*randn(1));
    logproppdf_alpha=@(x,y) log(normpdf(x,y,prop_std_alpha)+normpdf(x,-y,prop_std_alpha));
    [s_alpha accp_alpha]=mhsample(s_alpha,1,'logpdf',logpdf_alpha,'proprnd',proprnd_alpha,'logproppdf',logproppdf_alpha);
    
    
    %% record samples
    n1=floor(n/thin);
    if n1==n/thin
        samples_beta(:,:,n1)=s_beta;
        accp_matrix_beta(:,:,n1)=cell2mat(sample_accp_beta');
        samples_lambda(:,:,n1)=s_lambda;
        samples_v(1,n1)=s_v;
        samples_d(:,n1)=s_d;
        samples_alpha0(1,n1)=s_alpha0;
        accp_matrix_alpha0(1,n1)=accp_alpha0;
        samples_alpha(1,n1)=s_alpha;
        accp_matrix_alpha(1,n1)=accp_alpha;
    end
end
avg_accp=mean(accp_matrix_beta,3);
%% compare fitting error of beta between ML and Bayesian
burnin=floor(nsamples/2)+1;
beta_e1=mean(samples_beta(:,:,burnin:end),3);
d_e=mean(samples_d(:,burnin:end),2);

save(strcat(Result_dir,'/result_isof',num2str(K),'.mat'),'-v7.3')