function [rTP, rFP,auc,PR_RC_F] = func_Eval_Perf(pval, d,THRD,IND)
% [rTP, rFP] = plot_ROC(rank, GT_genes)
% ROC curve plot
% Input: 
%  pval: estimates of the genes
%  GT_genes: ground-truth statues
% Output:
%  rTP: true positive rate
%  rFP: false positive rate
% precision: precision at the THRD
% recall: recall at the THRD
% fscore: fscore at the THRD

GT_genes = find(d==1);
if strcmp(IND,'pval')
[~,rank] = sort(pval,'ascend');
elseif strcmp(IND,'prob')
   [~,rank] = sort(pval,'descend');
end

%% calculate ROC
ngenes = length(pval);
nGT = length(GT_genes);
tFP = ngenes - nGT;

rTP = zeros(1,ngenes);
rFP = 1:ngenes;
[a b] = intersect(rank, GT_genes);
b = sort(b);
count = 0;
for i=1:nGT
    count = count + 1;
    if i==1
        start = 1;
    else 
        start = b(i-1);
    end
    
    send = b(i)-1;
    
    rTP((send+1):ngenes) = count;
    
    if send==0
        rFP(start)=0;
        rFP((start+1):ngenes) = 1:(ngenes-count);
    else
        t = rFP(send);
        rFP((send+1):ngenes) = t:(ngenes-count);
    end
end
rTP = rTP./nGT;
rFP = rFP./tFP;

auc=sum(diff(rFP(1:end)).*rTP(2:end));
%% calculate precision recall fscore
if strcmp(IND,'pval')
precision = sum(pval<THRD & d==1)/sum(pval<THRD);
recall = sum(pval<THRD & d==1)/sum(d==1);
elseif strcmp(IND,'prob')
    precision = sum(pval>THRD & d==1)/sum(pval>THRD);
recall = sum(pval>THRD & d==1)/sum(d==1);
end

f_score = 2*precision*recall/(precision+recall);
PR_RC_F = [precision recall f_score];


